#!/usr/bin/php
<?php

if (!isset($argc) || is_null($argc))
    die('Not CLI mode');

$words = [];

chdir("HTML/");
foreach (glob("*.html") as $fn) {
    if (fnmatch("i_*",$fn)) continue;
    if (fnmatch("antau*",$fn)) continue;
    if (fnmatch("gramat*",$fn)) continue;
    if (fnmatch("index*",$fn)) continue;
    if (fnmatch("EoRu*",$fn)) continue;
    if (fnmatch("kovril*",$fn)) continue;
    if (fnmatch("elsxut*",$fn)) continue;
    if (fnmatch("reklam*",$fn)) continue;

    processFile($fn);
}

sort($words);

$f = fopen("index.ind","w+");

$oldword = '';
foreach ($words as $w) {
    if ($oldword == $w[0])
        fwrite($f," {$w[1]}{$w[2]}{$w[3]}");
    else if ($oldword != '')
        fwrite($f,"\n{$w[0]} {$w[1]}{$w[2]}{$w[3]}");
    else
        fwrite($f,"{$w[0]} {$w[1]}{$w[2]}{$w[3]}");
    $oldword = $w[0];
}
fwrite($f,"\n");

function processFile($fname) {
    global $words;

    $excludeWords = ["о","как","и","за","делать","сделать","из","к","быть","не","для","с","по","на","в"];
    $f = file_get_contents($fname);
    $fn = preg_replace('/\.html$/','',$fname);
    $end = 0;
    for (;;) {
        list($start,$end,$def,$tr,$com,$ex) = getItem($f,$end);
        if ($start == 0) break;
        
        foreach ($def as $w)
            $words[] = [$w, '1', $fn, $start];
        foreach ($tr as $w)
            $words[] = [$w, '2', $fn, $start];
        foreach ($com as $w) {
            if (in_array($w, $excludeWords)) continue;
            $words[] = [$w, '3', $fn, $start];
        }
        foreach ($ex as $w) {
            if (in_array($w, $excludeWords)) continue;
            $words[] = [$w, '4', $fn, $start];
        }
    }
}

function getDL($f, $offset = 0) 
{
    $start = strpos($f, '<dl>', $offset);
    if ($start === false)
        return [0, '', 0];
    $end = strpos($f, '</dl>', $start + 1);
    if ($end === false)
        $end = strlen($f);
    return [$start, $end+5, substr($f, $start, $end - $start + 5)];
}

function clean($s)
{
    $s = preg_replace('!<dl>!','',$s);
    $s = preg_replace('!</dl>!','',$s);
    $s = preg_replace('!<em>!','',$s);
    $s = preg_replace('!</em>!','',$s);
    $s = preg_replace('!<br( /)?>!','',$s);
    $s = preg_replace('!<span\s+.*?>!','',$s);
    $s = preg_replace('!</span>!','',$s);
    $s = preg_replace('!<a\s+.*?>!','',$s);
    $s = preg_replace('!</a>!','',$s);
    $s = preg_replace('!<ol>!','',$s);
    $s = preg_replace('!</ol>!','',$s);
    $s = preg_replace('!<li>!','',$s);
    $s = preg_replace('!</li>!','',$s);
    $s = preg_replace('!<b>!','',$s);
    $s = preg_replace('!</b>!','',$s);
    $s = preg_replace('!<sup>!','',$s);
    $s = preg_replace('!</sup>!','',$s);
    $s = preg_replace('!<sub>!','',$s);
    $s = preg_replace('!</sub>!','',$s);
    $s = preg_replace('!<tt>!','',$s);
    $s = preg_replace('!</tt>!','',$s);
    $s = preg_replace('!#!','',$s);
    $s = preg_replace('!\+!','',$s);
    $s = preg_replace('!\*!','',$s);
    $s = preg_replace('!=&gt;!','',$s);
    $s = preg_replace('!&ndash;!','',$s);
    $s = preg_replace('!&mdash;!','',$s);
    $s = preg_replace('!&laquo;!','',$s);
    $s = preg_replace('!&raquo;!','',$s);
    $s = preg_replace('!&quot;!','',$s);
    $s = preg_replace('!=!','',$s);
    //TODO убирать (= ...) (=> ...) ???

    $s = preg_replace('!\d!','',$s);
    $s = preg_replace('!"!','',$s);

    $s = preg_replace('!-!','',$s);
    $s = preg_replace('!,!','',$s);
    $s = preg_replace('!\.!','',$s);
    $s = preg_replace('!\?!','',$s);
    $s = preg_replace('/!/','',$s);
    $s = preg_replace('/:/','',$s);

    $s = preg_replace('!·!u','',$s);
    $acute = mb_convert_encoding('&#769;', "UTF-8", "HTML-ENTITIES");
    $s = preg_replace("!$acute!u",'',$s);
    $s = preg_replace('!;!','',$s);

    $s = preg_replace('/\s+/s',' ',$s);
    return $s;
}

function parse($s)
{
    $s = clean($s);
    $def = getList('!<dt>(.*?)</dt>!',$s);
    $tr = getList('!<dd>(.*?)</dd>!',$s);
    $com = getList('!<i>(.*?)</i>!',$s);
    $ex = getList('!<small>(.*?)</small>!',$s);

    return [$def,$tr,$com,$ex];
}

function getList($pat,$s)
{
    $res = '';
    $s = preg_replace_callback($pat,function($m) use(&$res) {
        $res .= $m[1] . ' ';
        return $m[0];
    },$s);
    $res = preg_replace('/\s+/',' ',$res);
    $res = preg_replace('!<i>.*?</i>!','',$res);
    $res = preg_replace('!<small>.*?</small>!','',$res);
    $res = preg_replace('!/!','',$res);

    //разворачиваем скобки
    $res = preg_replace('!(\w*)\((.*?)\)(\w*)!u','$1$3 $1$2$3',$res);

    $res = preg_replace('!\(|\)!','',$res);
    $res = preg_replace('!\s+!',' ',$res);
    $res = mb_convert_case($res,MB_CASE_LOWER,'utf-8');
    $res = preg_replace("!ё!u",'е',$res);
    
    $res = trim($res);
    $r = preg_split('/ /',$res);
    $r = array_unique($r);
    if ($r == [''])
        $r = [];
    return $r;
}

function getItem($f,$pos)
{
    list($start,$end,$dl) = getDL($f,$pos);
    list($def,$tr,$com,$ex) = parse($dl);
    return [$start,$end,$def,$tr,$com,$ex];
}
